package Scode; 

#
# Scode.pm 
#
# Version 0.91 1997/11/22
#
# Quick & Dirty Implementierung des SCODE-Algorithmus von
# Stefan Scheller in Perl fr das wmm.pl Script von Ramon Wartala
#
# Dieses Modul basiert auf der Idee und dem JavaScript-Quellcode
# von Stefan Scheller (mit Genehmigung des Autors). 
# Untersttzt werden die einfache Codierung von Zeichenketten ohne
# Sonderzeichen. Sowohl Abkrzungen vom Grad 5 als auch die Codierung 
# 4-stelliger Zahlen im Text werden in dieser Version nicht untersttzt.
#
# Bei Kritik und Anregungen zu Scode.pm bitte an Ramon Wartala wnden.
# Fragen zu SCODE bitte an Stefan Scheller.
#
# Mehr Informationen ber SCODE bei Stefan Scheller unter:
#   http://cips02.physik.uni-bonn.de/ScienceSite/scode
#
# (c) 1997, Ramon Wartala (wartala@stud.fh-heilbronn.de)
# (c) von SCODE bei Stefan Scheller
#

require Exporter;
use strict;

sub isLetter {
  my($letter) = @_;

  return(index 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ',$letter);
}

sub isNumber {
  my($number) = @_;

  return (index '0123456789',$number);
}

sub numberOf {
  my($chr,$string) = @_;
  my $j = 0;
  my $i = 0;
  my $slength = length $string;
 
  for ($i = 0; $i < $slength;$i++) {
    if ((chop $string) eq $chr) {$j++;}
  }

  return $j;
}

sub abk {
  my($text,$grad,$strich) = @_;
  
  # einfache Abkrzungen
  if($grad == 1) {
    $text=&replaceString('Th','T',$text);
    $text=&replaceString('th','t',$text);
    $text=&replaceString('ph','f',$text);
    $text=&replaceString('Ph','f',$text);
    $text=&replaceString('','s',$text);
  }
  # doppelte Zeichen mit regulren Ausdrcken eliminieren
  elsif($grad == 2) {
    $text =~ tr/a-zA-Z//s;
  }
  # 'speak lound' Regel
  elsif($grad == 3) {
    $text=&replaceString('en','N',$text); 
    $text=&replaceString('er','N',$text); 
    $text=&replaceString('be','N',$text); 
    $text=&replaceString('de','N',$text); 
    $text=&replaceString('ef','F',$text); 
    $text=&replaceString('ge','G',$text); 
    $text=&replaceString('Ha','H',$text); 
    $text=&replaceString('ka','K',$text); 
    $text=&replaceString('el','L',$text); 
    $text=&replaceString('em','M',$text); 
    $text=&replaceString('pe','P',$text); 
    $text=&replaceString('es','S',$text); 
    $text=&replaceString('te','T',$text); 
    $text=&replaceString('we','W',$text); 
    $text=&replaceString('zet','Z',$text);

    if ($strich==1) {
      $text=&replaceString('Eins','1',$text);
      $text=&replaceString('zwei','2',$text);
      $text=&replaceString('drei','3',$text);
      $text=&replaceString('sex','6',$text);
      $text=&replaceString('acht','8',$text);
    }
  }
  elsif($grad==4) {
    $text=&replaceString('au','aa',$text); 
    $text=&replaceString('Eu','ee',$text);
    $text=&replaceString('ion','ii',$text); 
  }

  return($text);
}

sub replaceString {
  my($oldS,$newS,$fullS) = @_;
  my $fullSlength=length $fullS;
  my $oldSlength=length $oldS;
  my $i=0;
  my $text = '';

FOUND: for (;;) {
    $i= index $fullS, $oldS,$i;
    if($i == -1) {last FOUND;} # last wirkt in Perl wie ein break in C;
    $fullS=(substr $fullS,0,$i).$newS.(substr $fullS,$i+$oldSlength);
  }
  return $fullS;
}

sub toStandard {
  my($text) = @_;
  my $test = '';
  my $textlength = 0;
  my $i = 0;
  my $found1 = -1;
  my $found2 = -1;
  my $tmptext = '';
  my $text1 = '';
  my $text2 = '';
  my $j = 0;

  $text = lc $text;
  $text = &replaceString('','ae',$text);
  $text = &replaceString('','oe',$text);
  $text = &replaceString('','ue',$text);
  $text = &replaceString('','ae',$text);
  $text = &replaceString('','oe',$text);
  $text = &replaceString('','ue',$text);
  $text = &replaceString('','ss',$text);

  $text =~ s/(\W)//g; # delete if char is not alphanumerical

  return $text; 
}

sub enscode {
  my($text,$power) = @_;
  my $text2 = '';

  # optimize -ung, -ueb, ...
  $text = toStandard($text);

  if((index $text,'ung') >= 0) {
    $text = replaceString('bung','01105',$text);
    $text = replaceString('dung','03105',$text);
    $text = replaceString('fung','04105',$text);
    $text = replaceString('gung','05105',$text);
    $text = replaceString('kung','08105',$text);
    $text = replaceString('mung','09105',$text);
    $text = replaceString('zung','17105',$text);  
  }
  if((index $text,'ueb') >=0) {
    $text = replaceString('bueb','01103',$text);
    $text = replaceString('dueb','03103',$text);
    $text = replaceString('fueb','04103',$text);
    $text = replaceString('gueb','05103',$text);
    $text = replaceString('kueb','08103',$text);
    $text = replaceString('mueb','09103',$text);
    $text = replaceString('zueb','17103',$text);
  }
  
  # now encode extended codes
  $text = replaceString('bu','000',$text);
  $text = replaceString('du','001',$text);
  $text = replaceString('fo','002',$text);
  $text = replaceString('fu','003',$text);
  $text = replaceString('gu','004',$text);
  $text = replaceString('ho','005',$text);
  $text = replaceString('ko','006',$text);
  $text = replaceString('ku','007',$text);
  $text = replaceString('mo','008',$text);
  $text = replaceString('mu','009',$text);
  $text = replaceString('om','100',$text);
  $text = replaceString('pf','101',$text);
  $text = replaceString('po','102',$text);
  $text = replaceString('ueb','103',$text);
  $text = replaceString('um','104',$text);
  $text = replaceString('ung','105',$text);
  $text = replaceString('vo','106',$text);
  $text = replaceString('wo','107',$text);
  $text = replaceString('zu','108',$text);
  $text = replaceString('zw','109',$text);

  # encode basic codes
  $text = replaceString('sch','13',$text);
  $text = replaceString('ich','07',$text);
  $text = replaceString('ch','02',$text);

  if($power == 1) {
    $text = replaceString('c','185',$text);
    $text = replaceString('j','186',$text);
    $text = replaceString('qu','187',$text);
    $text = replaceString('q','187',$text);
    $text = replaceString('x','188',$text);
    $text = replaceString('y','189',$text);
  }
  else {
    $text = replaceString('c','02',$text);
    $text = replaceString('j','07',$text);
    $text = replaceString('qu','13',$text);
    $text = replaceString('q','13',$text);
    $text = replaceString('x','17',$text);
    $text = replaceString('y','17',$text);
  }

  $text = replaceString('b','01',$text);
  $text = replaceString('d','03',$text);
  $text = replaceString('f','04',$text);
  $text = replaceString('g','05',$text);
  $text = replaceString('h','06',$text);
  $text = replaceString('k','08',$text);
  $text = replaceString('m','09',$text);
  $text = replaceString('o','11',$text);
  $text = replaceString('p','12',$text);
  $text = replaceString('u','14',$text);
  $text = replaceString('v','15',$text);
  $text = replaceString('w','16',$text);
  $text = replaceString('z','17',$text);

  $text = replaceString('a','2',$text);
  $text = replaceString('e','3',$text);
  $text = replaceString('i','4',$text);
  $text = replaceString('l','5',$text);
  $text = replaceString('n','6',$text);
  $text = replaceString('r','7',$text);
  $text = replaceString('s','8',$text);
  $text = replaceString('t','9',$text);
  
  return $text;
}


