#!/usr/bin/perl -w

use Net::SMTP;  # fuer SMTP- und 
use Net::POP3;  # POP3-Zugriff
use IniConf;    # zum Auslesen der .ini-Datei
use Scode;      # fuer die Codierung der Nachricht auf numerischen Pagern

# globale Variablen vorwrts deklarieren
use vars qw(
  $adr $serv $len $code $sender $mailm 
  $mailspool $pop3 @pop3row $no_of_names @liste
);

my $subject = '';             # Nachricht
my @pagertime = (9,12,16,21); # in dieser Zeit will ich erreichbar sein
my $calltime = 0;             # Feldindex fuer @pagertime
my @moment = ();              # Feld zur Aufnahme der aktuellen Zeit

# Konfigurationsdatei wmm.ini lesen...
&read_ini;

# Endlosschleife fr nohup-Aufruf erzeugen...
for(;;) {
  @moment = localtime(time); # lokale Uhrzeit ermitteln
  foreach $calltime (@pagertime) {
    if($moment[2] == $calltime) { # Nachricht nur zur richtigen Zeit schicken
      # soll der Text oder SCODE codiert werden...
      if($code eq 'TEXT') {
        $subject = "WMM:";
      } elsif($code eq 'SCODE') {
        $subject = '';
      }
      # wie komme ich an mein Postfach heran (POP3/mailspool)
      if($mailm eq 'POP3') {
        &get_pop3; # holt die E-Mails vom POP3-Server und s
                   # peichert sie in mailspl.txt
        &read_spoolfile;
      } elsif($mailm eq 'Mail') {
        &read_spoolfile;
      }
    }
  }
  sleep 3600 # eine Stunde schlafen legen...
}

#Konfigurationsdatei lesen
sub read_ini {
  my ($index, $array_idx, $wmm_ini) ;
  my $wmm_filename = "wmm.ini";

  # Prfen ob wmm.ini existiert und zum Lesen geffnet werden kann
  die "WMM: Leider keine wmm.ini-Datei gefunden!\n"
    unless (-e $wmm_filename && -r $wmm_filename);

  $wmm_ini = IniConf->new(-file =>$wmm_filename);
  
  $adr = $wmm_ini->val('EINSTELLUNGEN', 'PagerAdress');
  $serv = $wmm_ini->val('EINSTELLUNGEN', 'SMTPServer');
  $len = $wmm_ini->val('EINSTELLUNGEN', 'MessageLen');
  $code = $wmm_ini->val('EINSTELLUNGEN', 'Code');
  $sender = $wmm_ini->val('EINSTELLUNGEN', 'SenderAccount');
  
  # auf einfache Mail-Loops pruefen
  die "Absender- und Empfngeradresse sind gleich!\n" .
    "Abbruch wegen mglicher Mail-Loops." if($adr eq $sender);
  
  $mailm = $wmm_ini->val('EINSTELLUNGEN', 'GetMethode');
  
  if($mailm eq 'POP3') {
    $pop3 = $wmm_ini->val('EINSTELLUNGEN', 'POP3Server');
    @pop3row = split ' ',$pop3;
    $mailspool = "mailspl.txt";
  } elsif($mailm eq 'Mail') {
    $mailspool = $wmm_ini->val('EINSTELLUNGEN', 'MailBox');
  }
  
  # Liste mit den Namen einlesen und SCODE-Nr. erzeugen
  $no_of_names = $wmm_ini->val('NAMEN','Anzahl');
  $array_idx = 0;
  for($index = 1; $index <= $no_of_names; $index++) {
    $liste[$array_idx] = $wmm_ini->val('NAMEN','Name'.$index);
    $liste[$array_idx] .= " ".$index;
    $array_idx++;
  }
}

sub read_spoolfile {
  my (@filestatus, $mailfrom, $filesize, @ename, @subfield,
      $mailsubject, $name, $line, $new_subject, $abk,
      $namefound);

  my $counter = 0;
  # Trenner fuer einzelne Meldungen (: oder ,)
  my %sep = ( "[NB]" => ":", 
	      "[NN]" => "," );  

  # Mailspool-Datei existiert nicht oder ist nicht lesbar?
  # ENDE!
  die "WMM: Leider kann ich Ihr Postfach " .
    $mailspool." nicht oeffnen!\n" 
      unless ( -e $mailspool  &&  -r $mailspool);
  # Datei ist leer : Zurck
  return if (-z $mailspool);

  open(MAILSPOOL, $mailspool);
  
  while (defined($line = <MAILSPOOL>)) {
    chomp $line;
    if($line =~ /From:/) {
      $namefound = 0;
      $mailfrom = $';          # Alles nach dem From:
      foreach $name (@liste) {              # durchsuche Liste mit wichtigen Namen
        @ename = split ' ',$name;           # extrahiere die einzelnen Felder 
        if($mailfrom =~ /($ename[2])/) {    # ist die EMail-Adresse in der Liste ?
          $namefound = 1;
	  $counter++;  # Zhler erhhen
          if($code eq 'TEXT') {
            $subfield[$counter] = $ename[0] . $sep{$ename[3]};
          } elsif($code eq 'SCODE') {
            $subfield[$counter] = $ename[4];
          }
          last;  # Gefunden -- raus aus dem foreach
        }
      } # end foreach
    } elsif ($namefound && $line =~ /Subject:\s/ && 
      $ename[3] eq '[NB]') {
      # Name gefunden, Subject-Zeile gefunden, UND 
      # Subject soll bermittelt werden
      $subfield[$counter] .= $'; 
      if($code eq 'TEXT') {
        $subfield[$counter] .= ",";
      }
    }
  }

  close MAILSPOOL;

  # Wenn keine passende Mail gefunden
  # Ende
  return if ($counter == 0); 
    
  # Feld mit den einzelnen Betreffs umkehren, damit die neuen
  # immer vorne im Subject der Nachricht auftauchen 
  
  foreach (reverse @subfield) {
    $subject .= $_;
  }
  # Der letzte Schliff fr das Subject
  
  # ggfs. kuerzen + etc. anhaengen
  if($code eq 'TEXT') {
    # letztes Komma lschen
    $new_subject = substr $subject,0, -1;
    
    if((length $new_subject) > $len) {
      # Immer noch zu lang: 
      # alphanumerische Zeichen rauswerfen
      $new_subject =~ s/(\W)//g; 
      
      # immer noch zu lang? Abschneiden!
      if((length $new_subject) > $len) {
        substr($new_subject, $len-3) = "...";
      }
    }
  } elsif ($code eq 'SCODE') {
    $new_subject = Scode::enscode($subject,0);
    my $i = 1;
    # sukzessive strker komprimieren
    while ((length $new_subject) > $len && $i < 5) {
      $abk = Scode::abk($subject, $i, 0);
      $new_subject = Scode::enscode($abk,0);
      $i++;
    }
    # Immer noch zu lang? "999" anhngen !
    substr($new_subject, $len -3) = "999" 
      if (length $new_subject > $len);
  }
  
  $subject = $new_subject; # ich habe mein Mglichstes versucht...

  # neue EMail nur verschicken wenn sie eine Nachricht enthlt
  if(($code eq 'TEXT') && ((length $subject) > 4)) {
    &send_to;
  } elsif (($code eq 'SCODE') && ((length $subject) > 0)) {
    &send_to;
  }
} 

sub get_pop3 {
  my $pop3_host = $pop3row[0]; # Name des POP3-Hosts
  my $pop3_user = $pop3row[1]; # Name des Benutzers
  my $pop3_pass = $pop3row[2]; # Benutzer Passwort
  my $no_of_msg = 0;           # Anzahl Nachrichten im Postfach
  my $arrayref;
  my $index = 0;
  my $msg_line = 0;

  # Verbindung zu POP3-Host aufbauen...
  my $pop3 = Net::POP3->new($pop3_host);
 
  # lokale Mailspooler-Datei anlegen 
  open(POP3SPOOL, '>mailspl.txt');

  if($pop3) {$pop3->user($pop3_user)}; # Benutzer Name...
  if($pop3) {
    $no_of_msg = $pop3->pass($pop3_pass); # Passwort uebertragen...
    for($index = 1;$index <= $no_of_msg;$index++) {
      $arrayref = $pop3->get($index); # Alle E-Mails holen
      $msg_line = 0;
      while (${$arrayref}[$msg_line]) {
        print POP3SPOOL ${$arrayref}[$msg_line],"\n"; # Zeilenweise speichern
        $msg_line++;
      }   
    }
    $pop3->quit(); # Verbindung zum POP3-Server beenden
  }
  
  close(POP3SPOOL);
}

sub send_to {
  my $smtp;

  $smtp = Net::SMTP->new($serv);
  $smtp->mail($sender);
  $smtp->to($adr);
  $smtp->data();
  $smtp->datasend("TO: ".$adr."\n");
  $smtp->datasend("Subject: ".$subject."\n");
  $smtp->datasend("From: ".$sender."\n");
  $smtp->datasend("\n");
  $smtp->datasend("Automatisch erstellt von WMM, " . 
		  "EMail-Benachrichtigungssystem ".
		  "von R. Wartala und C. Kirsch\n");
  $smtp->datasend("(c) 1997,1998 by R. Wartala","\n");
  $smtp->dataend;
  $smtp->quit;
}
